// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.awt.*;
import java.awt.font.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.text.AttributedString;
import java.text.AttributedCharacterIterator;
import javax.swing.*;
import javax.imageio.ImageIO;
import java.util.*;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;


public class AboutDialog extends JDialog{

    private static JPanel logo=null;

    public AboutDialog(MainWindow win, String title){
        super((Frame) win, title, true);
        ResourceBundle bundle=win.getBundle();        
        JPanel contentPane=new JPanel();
        contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
        
        if(logo==null){
            logo=new LogoPanel(bundle);
        }
        logo.setAlignmentX(Component.CENTER_ALIGNMENT);
        JButton jBtnOK=new JButton(bundle.getString("BUTTONOK"));
        jBtnOK.setAlignmentX(Component.CENTER_ALIGNMENT);
        contentPane.add(logo);
        contentPane.add(Box.createVerticalStrut(15));
        contentPane.add(jBtnOK);
        contentPane.add(Box.createVerticalStrut(10));
        jBtnOK.addActionListener(
                new ActionListener(){
                    public void actionPerformed(ActionEvent evt){
                        dispose();
                    }
        });

        setContentPane(contentPane);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        pack();
        setResizable(false);
        setLocationRelativeTo(win);
        setVisible(true);
    }
}

class LogoPanel extends JPanel{
    
    private static String LOGOFILE="logo.png";
    private static int MARGIN=5;
    private int jpwidth=400;
    private int jpheight=400;   
    private HashMap<TextAttribute, Object> fontstyle=null;
    private RenderingHints renderhints=null;
    private BufferedImage bufimg=null;
    private String announcement=null;
    

    public LogoPanel(ResourceBundle bundle){
        super(new BorderLayout());
        setBorder(BorderFactory.createLoweredBevelBorder());
        try{
            InputStream is=getClass().getResourceAsStream(LOGOFILE);
            if(is==null)
                throw new FileNotFoundException();
            else{
                bufimg=ImageIO.read(is);
                is.close();
                jpwidth=bufimg.getWidth();
                jpheight=bufimg.getHeight();
            }

            Map <RenderingHints.Key, Object> rh=null;
            rh=new HashMap<RenderingHints.Key, Object>();
            rh.put(RenderingHints.KEY_RENDERING,
                   RenderingHints.VALUE_RENDER_QUALITY);
            

            renderhints=new RenderingHints(rh);
        }catch(IOException io){
            io.printStackTrace(System.err);
        }

        fontstyle=new HashMap<TextAttribute, Object>();
        fontstyle.put(TextAttribute.FAMILY, Font.DIALOG);
        fontstyle.put(TextAttribute.FOREGROUND, Color.BLACK);
        fontstyle.put(TextAttribute.SIZE, new Float(10.0));
        fontstyle.put(TextAttribute.WEIGHT, TextAttribute.WEIGHT_MEDIUM);
        announcement=bundle.getString("ABOUT");
    }

    @Override
    public Dimension getPreferredSize(){
        return new Dimension(jpwidth, jpheight);
    }

    @Override
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2d=(Graphics2D) g;
        g2d.setRenderingHints(renderhints);
        Rectangle rect=new Rectangle(0, 0, jpwidth, jpheight);
        g2d.setPaint(new TexturePaint(bufimg, rect));
        Rectangle2D viewarea=new Rectangle2D.Float(0, 0, jpwidth, jpheight);
        g2d.fill(viewarea);        
        printAnnounce(g2d);
    }

    private void printAnnounce(Graphics2D g2d){
        java.util.List<TextLayout> lines=new LinkedList<TextLayout>();      
        AttributedString attstr=new AttributedString(announcement, fontstyle);
        AttributedCharacterIterator chrIt=attstr.getIterator();
        FontRenderContext frc = g2d.getFontRenderContext();
        LineBreakMeasurer lbm=new LineBreakMeasurer(chrIt, frc);

        TextLayout line=null;

        float parwidth=(float)(jpwidth-2*MARGIN);
        float posX=(float) MARGIN;
        float posY=0;
        int parend=chrIt.getEndIndex();
        int lfind=announcement.indexOf("\n");
        float texthgt=0;

        while(lbm.getPosition() < parend){

            if(lbm.getPosition() >= lfind){
                lfind=announcement.indexOf("\n", lfind+1);
                if(lfind<0)
                    lfind=parend;
            }

            line=lbm.nextLayout(parwidth, lfind, true);
            if(line!=null){
                lines.add(line);
                texthgt+=line.getAscent()+line.getDescent()+line.getLeading();
            }
        }

        posY=(float)jpheight-(texthgt+MARGIN);
        for(TextLayout textline: lines){
            posY+=textline.getAscent();
            textline.draw(g2d, posX, posY);
            posY+=textline.getDescent()+textline.getLeading();
        }
    }
}