// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.io.*;
import java.util.LinkedList;


public class MsgProcessor {
    private final static int BUFSIZE=4096;
    private final static byte END=0x0A;
    private final static byte ESCAPE=0x5C;
    private int freeindex=0;
    private InputStream input=null;
    private OutputStream output=null;
    private byte[] buffer=null;


    public MsgProcessor(InputStream in, OutputStream out){
        input=in;
        output=out;
        buffer=new byte[BUFSIZE];
    }

    private void shiftLeft(int from){
        for(int i=from; i<freeindex; i++)
            buffer[i-1]=buffer[i];
        
        freeindex--;
    }

    private int findMsgEnd(int from){

        while(from < freeindex && buffer[from] != END) from++;

        if(from < freeindex)
            return from;
        else
            return -1;
    }

    public byte[] readMsg() throws IOException{
        int nbytes=0, eolpos=-1, prev=0;
        boolean eol=false;
        byte[] aux;

        while(!eol && freeindex < BUFSIZE){
        
            eolpos=findMsgEnd(prev);
            while(eolpos>0 && buffer[eolpos-1]==ESCAPE){
                    shiftLeft(eolpos);
                    prev=eolpos;
                    eolpos=findMsgEnd(prev);
            }

            if(eolpos>0)
                eol=true;
            else{
                nbytes=input.read(buffer, freeindex, BUFSIZE-freeindex);
                if(nbytes>0){
                    prev=freeindex;
                    freeindex+=nbytes;
                }else{
                    eol=true;
                    eolpos=freeindex;
                }
            }
        }

        if(eol){
            aux=new byte[eolpos];
            System.arraycopy(buffer, 0, aux, 0, eolpos);
            if(freeindex > eolpos+1){   
                int i=0;
                for(i=0; i<freeindex-eolpos-1; i++)
                    buffer[i]=buffer[i+eolpos+1];

                freeindex=i;
            }else
                freeindex=0;

        }else{              
            aux=new byte[BUFSIZE];
            System.arraycopy(buffer, 0, aux, 0, BUFSIZE);
            freeindex=0;
        }

        return aux;
    }
    
    public void writeMsg(byte[] msg) throws IOException{
        writeMsg(msg, 0, msg.length);
    }

    public void writeMsg(byte[] msg, int offset, int length) throws IOException{
        LinkedList<Byte> newmsg=new LinkedList();

        for(int i=offset; i<offset+length; i++){
            if (msg[i]==END){
                newmsg.add(ESCAPE);
                newmsg.add(END);
            }else
                newmsg.add(msg[i]);
        }

        newmsg.add(END);
        byte[] data=new byte[newmsg.size()];
        int ind=0;
        for(Byte b:newmsg)
            data[ind++]=b.byteValue();

        output.write(data);
        output.flush();
    }

    public void release() throws IOException{
        output.close();
        output=null;
        input.close();
        input=null;
        buffer=null;
        freeindex=0;
    }
}